#!/home/ND/bin/perl

=head1 NAME

index2wordlist_idf.pl - compute IDF value for each word occured in indexes. Also generate a white space separted word list for ASPELL feeding

=head1 SYNOPSIS

perl index2wordlist_idf -i $CATEGORY

# compute keywords for all categories

perl index2wordlist_idf.pl -i all

# compute keywords for one category

perl index2wordlist_idf.pl -i articles

# compute keywords for multiple categories

perl index2wordlist_idf.pl -i "articles ebooks"

=head1 DESCRIPTION

This program compute the compute IDF value for each word occured in indexes. A word list is generated in order to feed into a dictionary generator (ASPELL). These computation are done by reading the indexes that are created using Plucene. Predefined stop words are filtered out in both computing.

=head1 OPTIONS

=over 4

=item B<-i>

The following categories are valid categories:

master articles ebooks journals library_science mathematics thesis_disertations medicine biological_science education teachers all

=back

=head1 FILES

=over 4

File Output

$category_list: a word list for ASPELL feeding

$category_df: a pre-compute idf value for each word occured in the index

File Input 

stopwords.txt: a list of stop words

=back

=head1 AUTHOR

Xiaorong Xiang <xxiang1@nd.edu>

=cut

use strict;
use Getopt::Std;
use Math::Complex;
use Plucene::Index::Reader;
use Carp;

use constant STOPWORDS_FILE => "stopwords.txt";

use constant CATEGORIES => "master articles ebooks journals library_science mathematics thesis_disertations medicine biological_science education teachers sheet_music earth";

# read stop words from the stopwords_file
my @stopwords = getStopWords();

# data directory

use constant DATA=>"/home/ND/mylibrary/htdocs/DATA";

# word list file
use constant WORDLIST=>"_list";

# idf file
use constant IDF=>"_df";

# count each word occurrence in the index

my %words_occur = ();

# total documents

my $total_documents;

my %option = ();
getopt("i", \%option);
if (!$option{i}) {
	croak "Usage: $0 -i <all, master, articles, ebooks, journals, library_science, mathematics, thesis_disertations, medicine, biological_science, education, teachers, earth>";
}
my $inputString = lc($option{i});
if ($inputString eq "all") {
	my @categories = split(/ /, CATEGORIES);
	foreach my $category (@categories) {
		 index2dump($category);
		 dump2wordlist($category . WORDLIST);
		 dump2idf($category . IDF);	  
	}
}
else {
	my @inputStrings = split(/ /, $inputString);
	foreach my $category (@inputStrings) {
		chomp($category);
		 if (CATEGORIES !~ m/$category/) {
 			 croak "The input category  $category is not exist";
	      	 }
		 else {
			 index2dump($category);
			 dump2wordlist($category . WORDLIST);	
			 dump2idf($category . IDF);	 
		 }
	 }		 
}	

# compute the word list and store in a file
sub dump2wordlist {
	my ($output) = @_;
	open(OUTPUT, ">$output")
		or die "Couldn't open $output for writing: $!\n";
			
	foreach my $word (keys %words_occur) {
				print OUTPUT "$word\n";
	}
	close(OUTPUT);
}

# compute the idf value and store in a file

sub dump2idf {
	
	my ($output) = @_;
	open(OUTPUT, ">$output")
		or die "Couldn't open $output for writing: $!\n";
	foreach my $word (keys %words_occur) {
		my $count = $words_occur{$word};
		my $factor = logn($total_documents/$count, 2);
		print OUTPUT "$word; $count; $factor\n";
	}
	close(OUTPUT);
}

# This is a subroutine modified from the dump_index script in Plucene package, filter out field and words we are not care about
sub index2dump {
	my ($indexfile) = @_;
	my $r     = Plucene::Index::Reader->open($indexfile);
	no strict 'refs';
	my @readers = (@{ $r->{readers} } ? @{ $r->{readers} } : $r);
	if (@readers == 1 and $r->isa("Plucene::Index::SegmentsReader")) {
	     	die "But no segments\n";
	}
	for my $reader (@readers) {
		my @terms = $reader->terms;
		my $td = $reader->term_docs;
		for my $t (@terms) {
			while ($t->next) {
				my $term = $t->term;
				# check the field that contains all the other fields
				if ($term->field eq "content") {
					if ((length($term->text) > 2) && (&isStopWord(word=>$term->text) == 0)) {			
						$td->seek($term);
						my ($docs, $freqs) = $td->read;
		   				for (0 .. $#$docs) {
							$words_occur{$term->text} += $freqs->[$_];	
	       					}
					}
				}
	 		}
       		}
	}
	$total_documents = $r->max_doc;
}

sub getStopWords {
	open(INPUT, "<".STOPWORDS_FILE)
		or die "Couldn't open stopwords.txt for reading: $!\n";
	my @stopwords = <INPUT>;
	close(INPUT);
	return @stopwords;
}
					

sub isStopWord{
	my %args = @_;
	my $word = $args{word};
	my $isStopWord = 0;  # is not a stop word
	foreach (@stopwords) {
		if ($_ eq $word) {
  			$isStopWord = 1;
 			last;
		}
	}
	return $isStopWord;
}
