#!/home/ND/bin/perl


=head1 NAME

keywords2mylibrary.pl - compute the keywords for each document


=head1 SYNOPSIS

perl keywords2mylibrary.pl -i $CATEGORY

# compute keywords for all categories

perl keywords2mylibrary.pl -i all

# compute keywords for one categorie

perl keywords2mylibrary.pl -i articles

# compute keywords for multiple categories

perl keywords2mylibrary.pl -i "articles ebooks"


=head1 DESCRIPTION

This program compute the keywords for each document using (TF/IDF) score. The first five keywords are stored into the mylibrary database. The program reads precomputed IDF value from a file which stores the value corresponding to each word. Also the program reads stop words from a file.


=head1 OPTIONS

=over 4

=item B<-i>  

The following categories are valid categories:

articles ebooks journals library_science mathematics thesis_disertations medicine biological_science education teachers all

=back


=head1 FILES

=over 4

File Input

$category_df: a pre-compute idf value for each word occured in the index

stopwords.txt: a list of stop words

=back


=head1 AUTHOR

Xiaorong Xiang <xxiang1@nd.edu>

=cut

# include the necessary modules
use MyLibrary::Core;
use strict;
use Plucene::Document;
use Plucene::Document::Field;
use Plucene::Index::Writer;
use Plucene::Analysis::SimpleAnalyzer;
use Time::localtime;
use Getopt::Std;
use Carp;

my %categories = (
	articles            => ['90358', '90348'],
	ebooks              => ['558296', '601724'],
	journals            => ['557411'], 
	library_science     => ['475690'],
	mathematics         => ['20648'],
	thesis_disertations => ['36'],
	medicine            => ['90353'],
	biological_science  => ['90351', '90353'],
	education           => ['2656'],
	earth				=> ['14419'],
	sheet_music         => ['840264'],
	teachers            => ['600609']
);

# define a number of constants
use constant CATEGORIES     => "articles ebooks journals library_science mathematics thesis_disertations medicine biological_science education teachers sheet_music earth";
use constant STOPWORDS_FILE => "stopwords.txt";
use constant DF_FILE        => "_df";

# The analyzer should be the same as your index
my $analyzer = "SimpleAnalyzer";

# read stop words from the stopwords_file
my @stopwords = getStopWords();

# store df values for each category that is read from file DF_FILE;
my %df = ();

# get the input
my %option = ();
getopt("i", \%option);
if (!$option{i}) { croak "Usage: $0 -i <, articles, ebooks, journals, library_science, mathematics, thesis_disertations, medicine, biological_science, education, teachers, sheet_music>" }
my $inputString = lc($option{i});

# branch accordingly
if ($inputString eq "all") {

	# process all categories
	my @categories = split(/ /, CATEGORIES);
	foreach my $category (@categories) {
	
		%df = openDf($category);
		createKeywords(@{$categories{$category}});
		
	}
	
}

else {

	# process one category
	my @inputStrings = split(/ /, $inputString);
	foreach my $category (@inputStrings) {
	
		chomp($category);
		if (CATEGORIES !~ m/$category/) { croak "The input category  $category is not exist" }
		else {
		
			%df = openDf($category);
			print @{$categories{$category}};
			createKeywords(@{$categories{$category}});
			
		}
		
	}
	
}


sub keywordsPerResource {

	my ($resource_id, $title, $description) = @_;
	my $keywords = &compute($title, $description);
	my $resource = MyLibrary::Resource->new (id => $resource_id);
	$resource->relation($keywords);	
	$resource->commit();
	
}


sub createKeywords {

	my $count = 0;
	my (@termids) = @_;
	my $tm = localtime;
	print "Start Indexing: ", $tm->hour, ":", $tm->min, ":", $tm->sec, " ", $tm->year+1900, "-", $tm->mon+1, "-", $tm->mday, "\n";

	# get the article term and all of the articles
	foreach my $termid (@termids) {
	
		my $term = MyLibrary::Term->new( id => $termid );
		my @resource_ids = $term->related_resources;
		my @resources = MyLibrary::Resource->get_resources(list => [@resource_ids], output => 'id');
	
		# process each article
		foreach my $resourceid (@resources) {
		
			my $resource = MyLibrary::Resource->new (id => $resourceid);
			my $resource_id = $resource->id;
			my $title = $resource->name;
			my $description = $resource->note;
			keywordsPerResource($resource_id, $title, $description);
			
		}
		
	}	

	my $tm = localtime;
	print "End Indexing: ", $tm->hour, ":", $tm->min, ":", $tm->sec, " ", $tm->year+1900, "-", $tm->mon+1, "-", $tm->mday, "\n";
	
}		


sub compute {

	my ($title, $description) = @_;
	#parse the text into a words
	my $class = "Plucene::Analysis::$analyzer";
	eval "require $class";
	my $c = $class->new;

	# give each description as input
	my $stream = $c->tokenstream({ field  => "dummy", reader => IO::Scalar->new(\$description) });
	my @data;
	push @data, $_->text while $_ = $stream->next;

	# give each title as input, words in title should have higher weight than words in description, the heuristic we use is count two times for words occured in title, this can be changed
	$stream = $c->tokenstream({ field  => "dummy", reader => IO::Scalar->new(\$title) });
	while ($_ = $stream->next) {
	
		push @data, $_->text;
		push @data, $_->text;
		
	}

	#count the word frequency given a text
	my %seen = ();
	foreach (@data) {
	
		if ((length($_) > 2 ) && (isStopWord(word=>$_) == 0)) { $seen{$_}++ }
		
	}

	my %tfidf = ();

	#compute the tfidf value
	foreach my $word (keys %seen) { $tfidf{$word} = $seen{$word} * $df{$word} }

	# sort the word based on the tfidf value, and output the first five
	my $count = 0;
	my $returnwords = "";
	foreach my $word (reverse sort { $tfidf{$a} <=> $tfidf{$b} } keys %tfidf) {
	
		if ($count == 0) { $returnwords = $word }
		elsif ($count < 5) { $returnwords = join(";", $returnwords, $word) }
		else { last }
		
		$count++;
		
	}
	
	return $returnwords;
	
}


#get the df value for a particular index
sub openDf{

	my ($category) = @_;
	my %df_per = ();
	my $inputfile = "<" . $category . DF_FILE;
	open(INPUT, $inputfile) or die "Couldn't open ebooks_df for reading: $!\n";
	while(<INPUT>) {
	
		my @line = split /; /, $_;
		chomp($line[0]);
		chomp($line[2]);
		$df_per{$line[0]} = $line[2];
		
	}
	
	close(INPUT);
	return %df_per;
	
}


sub getStopWords {

	open(INPUT, "<".STOPWORDS_FILE) or die "Couldn't open stopwords.txt for reading: $!\n";
	my @stopwords = <INPUT>;
	close(INPUT);
	return @stopwords;

}


sub isStopWord {

	my %args = @_;
	my $word = $args{word};
	my $isStopWord = 0;  # is not a stop word
	foreach (@stopwords) {
	
		chomp($_);
		if ($_ eq $word) {
		
			$isStopWord = 1;
			last;
			
		}
		
	}
	
	return $isStopWord;
	
}

