#!/home/ND/bin/perl


=head1 NAME

mylibrary2plucene - create index 


=head1 SYNOPSIS

perl mylibrary2plucene.pl -i $CATEGORY

# compute keywords for all categories

perl mylibrary2plucene.pl -i all

# compute keywords for one categorie

perl mylibrary2plucene.pl -i articles

# compute keywords for multiple categories

perl mylibrary2plucene.pl -i "articles ebooks"


=head1 DESCRIPTION

This program uses Plucene module to create index for each category. SimpleAnalyzer is used here. However, one can change it to any other sophisticated analyzer. Documents are read from the MyLibrary database.


=head1 OPTIONS

=over 4

=item B<-i>

The following categories are valid categories:

articles ebooks journals library_science mathematics thesis_disertations medicine biological_science education teachers all

=back


=head1 FILES

=over 4

File output

directory $category

=back


=head1 AUTHOR

Xiaorong Xiang <xxiang1@nd.edu>

=cut

use MyLibrary::Core;
use strict;
use Plucene::Document;
use Plucene::Document::Field;
use Plucene::Index::Writer;
use Plucene::Analysis::SimpleAnalyzer;
use Time::localtime;
use Carp;
use Getopt::Std;
	
my %categories = (
	articles            => ['90358', '90348'],
	ebooks              => ['558296', '601724'],
	journals            => ['557411'],
	library_science     => ['475690'],
	mathematics         => ['20648'],
	thesis_disertations => ['36'],
	medicine            => ['90353'],
	biological_science  => ['90351', '90353'],
	education           => ['2656'],
	sheet_music         => ['840264'],
	teachers            => ['600609'],
	earth               => ['14419']
);

use constant CATEGORIES => "articles ebooks journals library_science mathematics thesis_disertations medicine biological_science education teachers sheet_music earth";

# get the input
my %option = ();
getopt("i", \%option);
if (!$option{i}) { croak "Usage: $0 -i <all, articles, ebooks, journals, library_science, mathematics, thesis_disertations, medicine, biological_science, education, teachers, sheet_music>" }
my $inputString = lc($option{i});

# branch accordingly
if ($inputString eq "all") {

	# process each category
	my @categories = split(/ /, CATEGORIES);
	foreach my $category ( @categories ) { createindex( $category, @{ $categories{ $category } } ) }
	
}

else {

	# process one category
	my @inputStrings = split(/ /, $inputString);
	foreach my $category (@inputStrings) {
		chomp($category);
		if (CATEGORIES !~ m/$category/) { croak "The input category  $category is not exist" }
		else {
		
			print @{ $categories{ $category } };
  			createindex( $category, @{ $categories{ $category } } );
  			
    	}
    	
	}
	
}


sub createindex {

	my $count = 0;
	my ($indexname, @termids) = @_;

	# for log purpose
	my $tm = localtime;
	print "Start Indexing: ", $tm->hour, ":", $tm->min, ":", $tm->sec, " ", $tm->year+1900, "-", $tm->mon+1, "-", $tm->mday, "\n";

	# the third arguments for the writer, 1 -- create new writer, 0 -- add new documents to existing index
	my $writer = Plucene::Index::Writer->new($indexname, Plucene::Analysis::SimpleAnalyzer->new(), 1);	
	$writer->set_mergefactor(50);
	
	# get the article term and all of the articles
	foreach (@termids) {
	
		my $term = MyLibrary::Term->new( id => $_ );
		my @resource_ids = $term->related_resources;
		my @resources = MyLibrary::Resource->get_resources(list => [@resource_ids], output => 'id');
	
		# process each article
		foreach (@resources) {
		
			$count++;

			my $resource = MyLibrary::Resource->new (id => $_);
			my $doc = Plucene::Document->new();
			$doc->add(Plucene::Document::Field->Keyword(resourceid => $resource->id));
			$doc->add(Plucene::Document::Field->Text(title => $resource->name));
			$doc->add(Plucene::Document::Field->Text(creator => $resource->creator));		
			$doc->add(Plucene::Document::Field->Text(description => $resource->note));
			$doc->add(Plucene::Document::Field->Text(provider => $resource->publisher));
			$doc->add(Plucene::Document::Field->Text(date => $resource->date));
			my $url;
			foreach ($resource->resource_locations) {
			
				$url = $_->location;
				$doc->add(Plucene::Document::Field->Keyword(url => $url));
			}
			
			my @term_ids = $resource->related_terms;
			my $terms = '';
			foreach(@term_ids) {
			
				my $term = MyLibrary::Term->new(id => $_);
				$terms .= $term->term_name . '; ';
				
			}
			
			$doc->add(Plucene::Document::Field->Text(terms => $terms));
			
			# add an unstored field for default search without specify the term. Change the '' to ' ' 07/19/2005
			my $content = join(' ',$resource->name, $resource->creator, $resource->note, $terms, $url, $resource->publisher, $resource->date);
			$doc->add(Plucene::Document::Field->UnStored(content => $content));
			$writer->add_document($doc);
		
		}
		
	}	

	# for log purpose, end time of indexing each category
	my $tm = localtime;
	print "End Indexing: ", $tm->hour, ":", $tm->min, ":", $tm->sec, " ", $tm->year+1900, "-", $tm->mon+1, "-", $tm->mday, "\n";
	print $indexname, " size: ", $writer->doc_count, "\n";
	
	$writer->optimize;
	undef $writer;
	
}		
