#!/usr/bin/perl

# require the necessary tools
use Getopt::Std;
use MyLibrary::Core;
use Net::OAI::Harvester;
use strict;
use XML::Simple qw(:strict);
use constant METADATAPREFIX => 'oai_dc';
use constant LOCATION_TYPE  => 38;

$| = 1;

# Added for option to use different config
my %commandline_options = ();
getopts("c:", \%commandline_options);

# create a simple xml parser and parse the configuration file
my $xs = XML::Simple->new(ForceArray => 1, KeyAttr => ['sets', 'terms']);

#construct config obj
my $config;
if( defined( $commandline_options{c} ) ) {
	$config = $xs->XMLin( $commandline_options{c} );
}
else {
	$config = $xs->XMLin;
}

#Currently, XMLSimple croaks if there is an issue with the XML file
#If that changes, this should catch it. You can never have too much
#error checking
unless( defined( $config ) ) {
	print STDERR "Unable to parse config object.\nExiting\n";
	exit(1);
}

# extract the repositories and proces each one
my $repositories = $config->{repository};
for ( my $i = 0; $i <= $#{$repositories}; $i++ ) {

	my ( $repository, $url, $name, $from, $until, $max, @term_ids );

	$repository = $repositories->[$i]->{name};
	$url        = $repositories->[$i]->{url};
	
	my $sets = $repositories->[$i]->{sets};
	for ( my $i = 0; $i <= $#{$sets}; $i++ ) {
	
		my $set = $sets->[$i]->{set};
		for ( my $i = 0; $i <= $#{$set}; $i++ ) {
		
			$name  = $set->[$i]->{name};
			$from  = $set->[$i]->{from};
			$until = $set->[$i]->{until};
			$max   = $set->[$i]->{max};
			
			my $terms = $set->[$i]->{terms};
			for ( my $i = 0; $i <= $#{$terms}; $i++ ) {
			
				my $term = $terms->[$i]->{term};
				@term_ids = ();
				for ( my $i = 0; $i <= $#{$term}; $i++ ) { push @term_ids, $term->[$i]->{term_id} }
			
			}
			
			# echo the extracted data
			print "Repository (URL): $repository ($url)\n";
			print "\t name: $name\n";
			print "\t from: $from\n";
			print "\tuntil: $until\n";
			print "\t  max: $max\n";
			print "\t  ids: "; foreach (@term_ids) { print $_, ' ' }; print "\n\n";
						
			# create a harvester
			my $harvester = Net::OAI::Harvester->new( baseURL => $url );
			
			# create a set of options
			my %opts = ( 'metadataPrefix' => METADATAPREFIX );
			$opts{ 'from' }  = $from if $from;
			$opts{ 'until' } = $until if $until;
			$opts{ 'set' }   = $name if $name;
			
			# added them to the harvester and get the records
			my $records = $harvester->listRecords( %opts );
			
			# process each record
			my $finished = 0;
			my $counter = 0;
			while ( ! $finished ) { 
			
				# check for errors
				if ( $records->errorCode() ) {
				
					print STDERR $records->errorString();
					
					# if bad xml, print out the response
					if ( $records->errorCode() eq 'xmlParseError' ) { 
						open( FILE, $records->file() );
						print while ( <FILE> );
						close( FILE );
					}
					
					exit(1);
				}
			
				# process record
				while ( my $r = $records->next() ) { 
				
					if ($max) {
					
						# increment and check
						$counter++;
						if ($counter == $max + 1) { $finished = 1; last }
						
					}
					
					# extract what I want and save
					print $r->header()->identifier(), "\n";
					my $metadata = $r->metadata;
					my @urls = $metadata->identifier;
					my $url = '';
					foreach (@urls) { if (/^http:.+$/) { $url = $_; last } }
					next if (! $url);
					my $resource = MyLibrary::Resource->new;
					$resource->name($metadata->title);
					$resource->fkey($r->header()->identifier);
					$resource->creator($metadata->creator);
					$resource->note($metadata->description);
					$resource->add_location(location => $url, location_type => LOCATION_TYPE);
					$resource->related_terms(new => [@term_ids]);
					$resource->commit;
					
				}
			
				# resume?
				my $resumptionToken = $records->resumptionToken();
				if ( $resumptionToken ) {
				
					$opts{ resumptionToken } = $resumptionToken->token();
					print STDERR "using resuption token: ",$resumptionToken->token(),"\n";
					$records = $harvester->listRecords(resumptionToken => $resumptionToken->token());
					
				}
				
				else { $finished = 1 }
				
			}
		
		}
	
	}

}

=head1 oai2mylibrary.pl
oai2mylibrary.pl converts OAI repositories to MyLibrary resources.
See the sample config file, oai2mylibrary.xml for configuration options.
You can specify the config file with a flag of -c
Otherwise, the oai2mylibrary.xml is used.
=cut
