#!/usr/bin/perl

=head1 NAME

client.cgi - an SRU client

=head1 DESCRIPTION

This is a SRU client implementation. The query is accepted from a user's input and is convert to CQL syntax. A SRU SearchRetrive request is formed along with three x- parameters, x-database, x-spell, and x-thesarus. This request is sent to a SRU server. The response received from SRU server is an XML document. This document is rendered with a XML stylesheet and transformed to HTML. The client side implementation also includes the paging machanism for easy browsing.
 
=head1 AUTHOR

Xiaorong Xiang <xxiang1@nd.edu>

Eric Lease Morgan <emorgan@nd.edu>

=cut


# require/use
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use LWP::UserAgent;
use MyLibrary::Core;
use strict;
use XML::LibXML;
use XML::LibXSLT;

# define constants
use constant SRUROOT   => 'http://mylibrary.ockham.org/sru-server.cgi?operation=searchRetrieve&version=1.1&x-spell=1&x-thesarus=1';
use constant SRU2HTML  => 'etc/sru2html.xsl';
use constant TEMPLATE  => 'etc/template.txt';
use constant HOME      => 'etc/home.txt';
use constant SEARCH    => 'etc/search.txt';
use constant ABOUT     => 'etc/about.txt';
use constant FACETS    => 'etc/facets.txt';
use constant DB        => 'etc/indexes.db';
use constant RECORDS_PER_PAGE => 25;

# initialize
my $cgi = CGI->new;
my $xml;

# get the command
my $cmd = $cgi->param('cmd');

# branch accordingly; apparently i should be using CGI::Application here
if (! $cmd) {

	# display the home page
	$xml = &slurp(TEMPLATE);
	$xml =~ s/##CONTENT##/&slurp(HOME)/e;
		
}

elsif ($cmd eq 'about') {

	# display the about page
	$xml = &slurp(TEMPLATE);
	$xml =~ s/##CONTENT##/&slurp(ABOUT)/e;
	
}
		
elsif ($cmd eq 'facets') {

	# display the facet page
	$xml = &slurp(TEMPLATE);
	$xml =~ s/##CONTENT##/&slurp(FACETS)/e;
	$xml =~ s/##NUMBER_OF_ITEMS##/&number_of_items/e;
	$xml =~ s/##FACETS_AND_TERMS##/&facet_term_combinations/e;
	
}
		
elsif ($cmd eq 'search') {
	
	# get the input
	my $query    = $cgi->param('query');
	my $database = $cgi->param('idx');
	my $script   = $cgi->script_name;
	
	# slurp up the database and find the necessary data
	my %db = &read_db();
	my ($about, $name);
	foreach my $record ( values(%db) ) {
	
		if ( $record->{'key'} eq $cgi->param('idx') ) {
		
			$about = $record->{'description'};
			$name  = $record->{'name'}
		
		}
 		
	}
	
	# begin to complete the search results page
	$xml = &slurp(TEMPLATE);
	$xml =~ s/##CONTENT##/&slurp(SEARCH)/e;
	$xml =~ s/##SCRIPT##/$script/e;
	$xml =~ s/##INDEX##/$database/e;
	$xml =~ s/##QUERY##/$query/e;
	$xml =~ s/##NAME##/$name/e;
	$xml =~ s/##ABOUT##/$about/e;
			
	# no query supplied; therefore, no results
	if ( ! $query ) { $xml =~ s/##RESULTS##// }
	
	# process the query
	else {
	
		# do simple munging of the query; try to force it into CQL
		if ($query =~ /\s/) {
		
			if (($query =~ / and /) | ($query =~ / or /) | ($query =~ / not /)) { }		
			elsif ($query =~ /=/) { }
			elsif ($query !~ /"/) {
			
				# try to make queries with no syntactical sugar a bit "smarter"
				my @terms = split / /, $query;
				my $enhancement;
				for (my $i; $i <= $#terms; $i++) {
				
					if ($i < $#terms) { $enhancement .= $terms[$i] . ' and ' }
					else { $enhancement .= $terms[$i] }
				
				}
				
				$query = '"' . $query . '"' . " or ($enhancement)";
				
			}
			
		}

		# get the page number
		my $pageNumber;
		if ($cgi->param('pageNumber')) { $pageNumber = $cgi->param('pageNumber') }
		else { $pageNumber = 0 }
		my $startRecord = $pageNumber * RECORDS_PER_PAGE + 1;
		
		# create a user agent, create a request, send it, and get a response
		my $ua = LWP::UserAgent->new(agent => 'SRU-Client/0.1 ');
		my $request = HTTP::Request->new(GET => SRUROOT . '&query=' . $query . '&x-database=' . $database . '&startRecord=' . $startRecord . '&maximumRecords=' . ( RECORDS_PER_PAGE ) );
		my $response = $ua->request($request);
		
		# check response
		if ($response->is_success) {
		
			if ($cgi->param('output') eq 'HTML') {
			
				# prepare parameters for paging purpose, parameter for sru client
				my $clientParameters = "?cmd=search&" . "query=" . $query . "&output=HTML" . "&idx=" . $database; 
	
				# prepare parameters send to xslt except query, query need to replace by new query in xslt, sru client
				my $otherParameters = "?cmd=search" . "&output=HTML" . "&idx=" . $database;
	
				# replace the RESULTS token with the transformed SRU output
				$xml =~ s/##RESULTS##/&transform(xslt => SRU2HTML, xml => $response->content, pageNumber => $pageNumber, recordsPerPage => RECORDS_PER_PAGE, clientParameters => $clientParameters, otherParameters => $otherParameters)/e;

			}
			
			else { }
			
		}
		
		else { $xml = $response->status_line, "\n" }
	
	}
	
}

else {

	# error
	$xml  = $cgi->start_html(-title => 'SRU client front-end');
	$xml .= $cgi->h1('SRU client front-end');
	$xml .= $cgi->p("Unknown value for cmd ($cmd). Call Eric.");
	$xml .= $cgi->end_html;

}


# done
print $cgi->header(-type=>'text/html', -charset => 'UTF-8');
print $xml;


sub number_of_items {

	return scalar(MyLibrary::Resource->get_resources(output => 'id'));
	
}


sub facet_term_combinations {

	# get all the facets, build a list, and display it
	my @facets = MyLibrary::Facet->get_facets (sort => 'name');
	my $items;
	foreach (@facets) {
	
		# create a list of terms associated with each facet
		my $terms = '<ol>';
		foreach my $id ($_->related_terms(sort => 'name')) {
		
			my $term = MyLibrary::Term->new(id => $id);
			$terms .= '<li>' . $term->term_name . ' <span style="color: gray"> (' . scalar($term->related_resources) . ' items)</span></li>';
		}
		
		$terms .= '</ol>';
		
		# build the list
		$items .= $cgi->li('<span style="font-weight: bold">' . $_->facet_name, '</span> - ', $_->facet_note, $terms);
		
	}
	
	# done
	return $cgi->ol($items);
	
}


sub transform {

	# get the input
	my %opts             = @_;
	my $pageNumber       = $opts{pageNumber};
	my $clientParameters = $opts{clientParameters};
	my $otherParameters  = $opts{otherParameters};
	my $recordsPerPage   = $opts{recordsPerPage};
	
	# create parser and process input
	my $parser     = XML::LibXML->new;
	my $xslt       = XML::LibXSLT->new;
	my $source     = $parser->parse_string($opts{xml}) or croak $!;
	my $style      = $parser->parse_file($opts{xslt})  or croak $!;
	my $stylesheet = $xslt->parse_stylesheet($style)   or croak $!;
	my $results    = $stylesheet->transform($source, pageNumber=>$pageNumber, recordsPerPage=>$recordsPerPage, XML::LibXSLT::xpath_to_string(clientParameters=>$clientParameters), XML::LibXSLT::xpath_to_string(otherParameters=>$otherParameters)) or croak $!;

	# done
	return $stylesheet->output_string($results);
	
}


sub slurp {

	# open a file named by the input and return its contents
	my $f = @_[0];
	my $r;
	open (F, "< $f");
	while (<F>) { $r .= $_ }
	close F;
	return $r;

}


sub read_db {
	
	# read a tab-delimited file defined by DB and return it as a hash
	my %db = {};
	open ( INPUT, '< ' . DB );
	while ( <INPUT> ) {
	
		chop;
		my ($key, $name, $description) = split /\t/;
		my $index               = {};
		$index->{'key'}         = $key;
		$index->{'name'}        = $name;
		$index->{'description'} = $description;
		$db{$key}               = $index;
		
	}
	
	close INPUT;
	return %db;
	
}

