#!/usr/bin/perl

=head1 NAME

server.cgi - an SRU server

=head1 DESCRIPTION

This is a SRU server implementation. This server implements Explain and SearchRetrieve operations in SRW/U specification. Scan operation is not implemented at this stage. The SRU server expects a valid CQL query from the client side. This CQL query is convert into Plucene syntax and sent to Plucene search engine. Records are returned to the client side in Dublin Core schema. At the server side, extra responses including spell suggestions and synonyms of a word are also returned. If the query is a complicated query with boolean operation, all the terms are extract from the query and parsed to single words. The spell suggestions and synonyms are given for each words.  

=head1 AUTHOR

Xiaorong Xiang <xxiang1@nd.edu>

Eric Lease Morgan <emorgan@nd.edu>

=cut

# require the necessary modules
use CGI qw(-oldstyle_urls);
use CGI::Carp qw(fatalsToBrowser);
use CQL::Parser;
use SRU::Request;
use SRU::Response;
use SRU::Response::Term;
use Plucene::QueryParser;
use Plucene::Analysis::SimpleAnalyzer;
use Plucene::Search::HitCollector;
use Plucene::Document;
use Plucene::Document::Field;
use Plucene::Search::IndexSearcher;

#for retrieve the keywords
use MyLibrary::Core;

use Text::Aspell;
use Lingua::Wordnet;
use CQL::Parser;
use CQL::TermNode;

# separate the explain function into a separate module
use lib "/home/ND/mylibrary/htdocs";
use explain;

use strict;
use warnings;

# existing databases users can choose for searching
use constant DATABASES => "master articles ebooks journals library_science mathematics thesis_disertations medicine biological_science education teachers sheet_music earth";
use constant DEFAULT_DATABASE => "ebooks";	
use constant DATA_DIR => "./DATA/";

# default number of records should return if no maximumRecords defined
use constant NUMBER_RECORDS => 250;

# seach synonyms, nouns, verbs, adjectives or adverbs
use constant POSES => qw(n v a s r);

# possible x-spell and x-thesarus values;
use constant SCALE => qw(0 1 2 3);

# initlize the necessary objects
my $cgi      = CGI->new();
my $request  = SRU::Request->newFromCGI($cgi)          || &error;
my $response = SRU::Response->newFromRequest($request) || &error;

# count total hits that matche the query, regardless the maximumRecords, for return purpose, numberOfRecords 
my $totalHits = 0;

# define a globle variable that stores the parsed terms
@main::traversalterms = ();

# Can not get the extra data parameter work from SRU interface, so use the cgi interface to get around it Xiaorong

my $database = $cgi->param('x-database');
my $spell = $cgi->param('x-spell');
my $thesarus = $cgi->param('x-thesarus');

# check for type of response; explain
if ($response->type() eq 'explain') {
	
	# fill up the response's record 
	$response->record(SRU::Response::Record->new(recordSchema => 'http://explain.z3950.org/dtd/2.0/',
					
			recordData => explain->record()));												      
}

# scan
elsif ($response->type() eq 'scan') {
	
	$response->addDiagnostic(SRU::Response::Diagnostic->newFromCode(4, 'Scan operation is not supported at this stage.'));
	$response->asXML();
}

# search
elsif ($response->type() eq 'searchRetrieve') {
	my $extraResponseData;
	# parse the query and check it
	my $parser = CQL::Parser->new;
	eval { $parser->parse($request->query) };
	if ($@) {
		diagoMessage(code=>10, detail=>$@);
     	}

	# No database is defined in the request, set to default one
	if (!$database) {
		$database = DEFAULT_DATABASE;
	}
	if (DATABASES !~ m/$database/) {
		diagoMessage(code=>10, detail=>"the database $database is not support");
	}
	else {
		$database = DATA_DIR . $database;
	}
        my $node = $parser->parse($request->query);

	# find out how many records need to return
	my $numberofResults;
	if (defined($request->maximumRecords)) {
		if (($request->maximumRecords) < NUMBER_RECORDS) {
			$numberofResults = $request->maximumRecords;
		}
	}
	else {
		$numberofResults = NUMBER_RECORDS;
	}		

	# find out start record
	my $startRecord;
	if (defined($request->startRecord)) {
		$startRecord = $request->startRecord();
	}
	else {
		$startRecord = 1;
	}
	# search using Plucene with defined database and number of required records
	my @results = &search(node=>$node->toLucene(), database=>$database, numberofResults=>$numberofResults, startRecord=>$startRecord);
	
	# process each result
	for (my $i = 0; $i <= $#results; $i++) {
		# create a records object and add it to the responee
		my $record = SRU::Response::Record->new(recordSchema => 'info:srw/schema/1/dc-v1.1', recordData => $results[$i]);
		$response->addRecord($record);
	}

	$response->numberOfRecords($totalHits);
	
	# process each spell
	if ($spell) {
		$extraResponseData .= &spell2XML(node=>$node, database=>$database, scale=>$spell);
	}

	# process each thesarus
	if ($thesarus) {
		$extraResponseData .= &thesarus2XML(node=>$node, scale=>$thesarus);
	}
	$response->extraResponseData($extraResponseData);
}


# done; output the result
print $cgi->header('text/xml');
print $response->asXML(encoding=>"ISO-8859-1");

exit;

# Return the diagonostic message
sub diagoMessage {
	my %arg = @_;
	$arg{code} = 10 unless exists $arg{code};
	$arg{detail} = "Not support" unless exists $arg{code};
	$response->addDiagnostic(SRU::Response::Diagnostic->newFromCode($arg{code},$arg{detail}));
	print $cgi->header('text/xml');
	print $response->asXML();
	exit;
}

sub spell2XML {
	my %args = @_;
	my $node = $args{node};
	my $database = $args{database};
	my $scale = $args{scale};
	@main::terms = ();
	my $extraResponseData;
	my %spellresults = &spell(node=>$node, database=>$database, scale=>$scale);
	$extraResponseData .= "<spellset>";
	foreach my $resultkey (keys %spellresults) {
		my @value = @{$spellresults{$resultkey}};
		$extraResponseData .= "<spell>";
		$extraResponseData .= "<term>$resultkey</term>";
		$extraResponseData .= "<found>$value[0]</found>";
		shift @value;
		$extraResponseData .= "<count>$value[0]</count>";
		shift @value;
		$extraResponseData .= "<suggestions>";
		foreach (@value) {
			$extraResponseData .= "<suggestion>$_</suggestion>";
		}
		$extraResponseData .= "</suggestions>";
		$extraResponseData .= "</spell>";
	}
	$extraResponseData .= "</spellset>";
	return $extraResponseData;
}

sub thesarus2XML{
	my %args = @_;
	my $node = $args{node};
	my $scale = $args{scale};
	@main::terms = ();
	my $extraResponseData;
	my %thesarusresults = &thesarus(node=>$node, scale=>$scale);
	$extraResponseData .= "<synonymset>";
	foreach my $resultkey (keys %thesarusresults) {
		my @value = @{$thesarusresults{$resultkey}};
		$extraResponseData .= "<synonym>";
		$extraResponseData .= "<term>$resultkey</term>";
		$extraResponseData .= "<count>$value[0]</count>";
		shift @value;
		$extraResponseData .= "<suggestions>";
		foreach (@value) {
			$extraResponseData .= "<suggestion>$_</suggestion>";
		}
		$extraResponseData .= "</suggestions>";
		$extraResponseData .= "</synonym>";
	}
	$extraResponseData .= "</synonymset>";
	return $extraResponseData;
}	

sub search {
	# initialize
	my %args = @_;
	my $query = $args{node};
	my $database = $args{database};
	my $numberofResults = $args{numberofResults};
	my $startRecord = $args{startRecord};
	my @results;
 	my @docs;	
	my $count = 0;

	# initialize the totalHits
	
	$totalHits = 0;
	
	# open the index to lucene;
	my $lucene = Plucene::QueryParser->new({analyzer => Plucene::Analysis::SimpleAnalyzer->new(), default => "content"}); # The default query is content which include everything
	my $lucenequery = $lucene->parse($query);
	my $lucenesearcher = Plucene::Search::IndexSearcher->new($database);
	# start searching process in Plucene, store results in @docs
	my $hc = Plucene::Search::HitCollector->new(collect => sub{
	    		my ($self, $doc, $score)= @_;		
	      		my $res = eval{$lucenesearcher->doc($doc)};
			if ($res) {
				$totalHits++;
				$count++;
				if (($count < ($numberofResults + $startRecord)) && ($count >= $startRecord)){
					push @docs, $res;
				}
				# loop through in order to get number of totalHist for returning correct numberOfRecords ???? 08/10
				# 
				#	else {
				#	last;
				#}
			}
				
		});
	$lucenesearcher->search_hc($lucenequery, $hc);
	$lucenesearcher->close;
	# create a map to get the reulsts from @docs
	my @mapdoc = map{{
                resourceid => $_->get("resourceid")->string,
                title => $_->get("title")->string,
                creator => $_->get("creator")->string,
		description => $_->get("description")->string,
		date => $_->get("date")->string,
		url => $_->get("url")->string,
		terms => $_->get("terms")->string,
		provider => $_->get("provider")->string,
        }} @docs;
	
	my $hits = $#docs+1;
	if ($hits > 0) {
		foreach (@mapdoc) {
			my $resource = MyLibrary::Resource->new(id => ($_->{resourceid}));
			my $keywords = $resource->relation;
			# push on to results
			push @results, '<srw_dc:dc xmlns="http://www.w3.org/TR/xhtml1/strict" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:srw_dc="info:srw/schema/1/dc-v1.1"><dc:title>' . &escape_entities($_->{title}) . '</dc:title><dc:identifier>' . &escape_entities($_->{url}) . '</dc:identifier><dc:description>' . &escape_entities($_->{description}) . '</dc:description><dc:creator>' . &escape_entities($_->{creator}) . '</dc:creator><dc:provider>' . &escape_entities($_->{provider}) . '</dc:provider><dc:date>' . &escape_entities($_->{date}) . '</dc:date><dc:relation>' . $keywords . '</dc:relation></srw_dc:dc>'; 
		}
	}
			
	# done; return it
	return @results;
} 

# For spell checking and suggestions, xiaorong 06/21/2005

sub spell {
	my %args = @_;
	my $node = $args{node};
	my $database = $args{database};
	my $scale = $args{scale};
	# First step is parsing the query to terms (single or phrase)
	# Then check if these terms (single or phrase) can be found in a dictionary
	# if term (single or phrase) can not be found,
	# if the term is single, display suggestions
	# else if the term is phrase, parse to single term, and check separately
	my @terms = &traversal($node);
	my %results;
	my $speller = Text::Aspell->new;
	croak("Can not create Aspell instance") unless $speller;
	my $dictionary = $database . '_dict';
	$speller->set_option('master', $dictionary);
	foreach my $term (@terms) {
		if ($speller->check($term)) {
			push(@{$results{$term}}, "1"); #found in dictionary
		}
		else {
			push(@{$results{$term}}, "0");
		}
		my @suggestions = $speller->suggest($term);
		my $total = &scaler(total=>$#suggestions, scale=>$scale);
		my @suggestwords;
		my $count = 0;

		# get rid of the duplicate value
		my %seen = ();
		foreach my $word (@suggestions) {
			if (($word lt $term) || ($word gt $term)) {
				push(@suggestwords, $word) unless $seen{$word}++;
				$count++;
			}
			if ($count > $total) {
				last;
			}
		}
		push (@{$results{$term}}, $count);
		push (@{$results{$term}}, @suggestwords);
	}
	return %results;
}

# return the number of spell should be retured
sub scaler {
	my %args = @_;
	my $scale = $args{scale};
	my $total = $args{total};
	
	if ($scale eq '0') {
		$total = 0;
	}
	elsif ($scale eq '1') {
		$total = 0.3 * $total;
	}
	elsif ($scale eq '2') {
		$total = 0.6 * $total;
	}
	return $total;
}

# traverse the tree to get terms, using a global variable to keep the terms 
	
sub traversal {
	my ($node) = @_;
	if ($node->isa('CQL::BooleanNode')) {
		traversal($node->left());
		traversal($node->right());
	}
	elsif ($node->isa('CQL::TermNode')) {
		my $term = $node->getTerm();
		push(@main::terms, $term);	
	}			
	return @main::terms;	
}

# thesarus, get synonyms
sub thesarus {
	my %args = @_;
	my $node = $args{node};
	my $scale = $args{scale};
	my $wn = new Lingua::Wordnet;
	my @terms = &traversal($node);	
	my %results;
	foreach my $term (@terms) {
		my @words;
		my $count = 0;
		# get rid of duplicate value
		my %seen = ();
		foreach my $pos (POSES) {
			my @synsets = $wn->lookup_synset($term, $pos);
			my $total = &scaler(total=>$#synsets, scale=>$scale);
			my $synsetcount = 0;
  				foreach my $synset (@synsets) {
					my $synword;
					if ($synsetcount > $total) {
						last;
					}
					else {
						$synsetcount++;
					}
					foreach $synword ($synset->words) {
						$synword =~ s/\%\d+$//;
  		 				$synword =~ s/\_/ /g;
						if (!($term eq $synword)) {
							push (@words, $synword) unless $seen{$synword}++;
							$count++;
						}
					}
				}
		}
		push (@{$results{$term}}, $count);
		push (@{$results{$term}}, @words);
	}
	return %results;
}

sub escape_entities {

	# get the input
	my $s = shift;
	
	# escape
	$s =~ s/&/&amp;/g;
	$s =~ s/</&lt;/g;
	$s =~ s/>/&gt;/g;
	$s =~ s/"/&quot;/g;
	$s =~ s/'/&apos;/g;

	# done
	return $s;
	
}


sub error {

	print $cgi->header();
	print $SRU::Error;
	exit;
	
}

