package MyLibrary::Patron;

use MyLibrary::DB;
use Carp qw(croak);
use strict;


=head1 NAME

MyLibrary::Patron

=head1 SYNOPSIS

	# require the necessary module
	use MyLibrary::Patron;

	# create an undefined Patron object
	my $patron = MyLibrary::Patron->new();

	# set the various attributes of a Patron object
	$patron->name_first('Robert');
	$patron->name_last('Fox');
	$patron->email('rfox2@nd.edu');
	$patron->term_id(25);

	# commit a Patron to the database
	$patron->commit();

	# get a list of Patron objects
	my @patrons = MyLibrary::Patron->get_patrons();

	# delete a Patron object from the database
	$patron->delete();

=head1 DESCRIPTION

Use this module to get and set patron information to a MyLibrary database as well as retrieve a list of all Patron objects in a MyLibrary instance. This package also contains several methods which can be used to retrieve related information about a given patron such as which resources they have selected as well as their customized interface.

=head1 METHODS

=head2 new()

This class method is the constructor for this package. The method is responsible for initializing all attributes associated with a given Patron object. The method can also be used to create a Patron object using a patron id or name. The patron would thus already need to exist in the database in order for these parameters to have any effect.

=head2 patron_id()

This method is used exclusively to retrieve an exising patron's database id, if the patron has been commited to the database. This method may not be used to set a patron's database id.

	# get patron id
	my $patron_id = $patron->patron_id();

This is a required Patron object attribute.

=head2 name_first()

This method may be used to either get or set a patron's first name. This is a required attribute, meaning that the object cannot be commited to the database if this attribute is left null.

	# set name_first()
	$patron->name_first('Robert');

	# get name_first()
	my $patron_first_name = $patron->name_first();

=head2 name_last()

This method may be used to either get or set a patorn's last name. This is a required attribute, meaning that the object cannot be commited to the database if this attribute is left null.

	# set name_last()
	$patron->name_last('Miller');

	# get name_last()
	my $patron_last_name = $patron->name_last();

=head2 email()

This method gets or sets a patron's email address. This is not a required attribute.

	# set patron's email address
	$patron->email('eric');

	# get patron's email address
	my $email = $patron->email();

=head2 page_title()

This method will get or set the title to the patron's customized page. This is not a required attribute.

	# set patron's page_title()
	$patron->page_title('eric');

	# get patron's page_title()
	my $page_title = $patron->page_title();

=head2 term_id()

This method can be used to retrieve or set the discipline term id with which the patron is associated. This is a required attribute, meaning that the object cannot be commited to the database if this attribute is left null.

	# set patron's term_id()
	$patron->term_id(25);

	# get patron's term_id()
	my $patron_term_id = $patron->term_id();

=head2 telephone()

This method can be used to get or set a patron's telephone number. The input to this method will be checked for a specific pattern. The allowed patters are:

	# examples of allowable telephone number patterns (# represents a numeric digit)
	#######
	##########
	###-####
	###-###-####

This is not a required patron attribute.

	# set a patron's telephone number
	$patron->telephone(999-999-9999);

	# get a patron's telephone number
	my $patron_telephone_number = $patron->telephone();

=head2 zipcode()

This method can be used to get or set a patron's address zip code. The input to this method will be checked for a very specific pattern. The only allowable pattern is (# representing a numeric digit): #####. This is not a required attribute because MyLibrary does not require the patron to submit a valid address to use the system.

	# set a patron's zipcode
	$patron->zipcode(99999);

	# get a patron's zipcode
	my $patron_zipcode = $patron->zipcode();

=head2 address_1()

This methode should be used to set or get the first part of a patron's address information. Typically, this is a street address or building location. This is not a required attribute.

	# set a patron's address part one
	$patron->address_1('2634 Willow Street');

	# get a patron's address part one
	my $patron_address_one = $patron->address_1();

=head2 address_2()

This method should be used to set or get the second part of a patron's address information. Typically, this is a street address. This is not a required attribute similar to address_1().

	# set a patron's address part two
	$patron->address_2('2634 Willow Street');

	# get a patron's address part two
	my $patron_address_two = $patron->address_2();

=head2 city()

This method should be used to set or get the city in which the patron lives. This is not a required attribute.

	# set a patron's city
	$patron->city('Rock Island');

	# get a patron's city
	my $patron_city = $patron->city();

=head2 state()

This method should be used to set or get the state in which the patron lives. This is not a required attribute. For non-USA countries, this could be the patron's province, etc.

	# set a patron's state
	$patron->state('Illinois');

	# get a patron's state
	my $patron_state = $patron->state();

=head2 can_contact()

This method should be used to set the can_contact flag. This is a binary attribute, and is not required. However, a devault value of '0' ('Do not contact') will be set if no value is indicated. The input to this method will be sanitized from non-binary content.

	# set a patron's can_contact flag
	$patron->can_contact(1);

	# get a patron's can_contact flag
	my $patron_contact_flag = $patron->can_contact();

=head2 password()

This method can be used to either retrieve or set a patron's password. This attribute will only be used when the system relies upon the 'default' method of authentication (which is to store patron passwords locally as opposed to relying upon an insitutional authentication system). The non-encrypted password chosen and entered by the patron will be encrypted. When the password is retrieved, it will also be in an encrypted form for security purposes. Authentication methods can then be used to perform password verification against this patron attribute. Alpha or numeric digits may be used in a patron's password in any order, however, authentication module methods may place certain requirements on password length and complexity. This method simply encrypts, stores and retrieves patron passwords.

	# set the patron's password
	my $entered_password = $input->{'password'};
	$patron->password($entered_password);

	# retrieve the encrypted form of a patron's password
	my $patron_password = $patron->password();

=head2 last_query()

This method will either set or get the last query performed by the patron in a Quick Search box on their customized page. This is not a required attribute.

	# set the patron's last Quick Search query
	$patron->last_query('influenza');

	# retrieve the patron's last Quick Search query
	my $patron_q_search_query = $patron->last_query();

=head2 wants_html()

This method should be used to set the wants_html flag, which indicates the type of output the patron desires from the current awareness service. This is a binary attribute, and is not required. However, a devault value of '0' ('Does not want html') will be set if no value is indicated. The input to this method will be sanitized from non-binary content.

	# set a patron's wants_html flag
	$patron->wants_html(1);

	# get a patron's wants_html flag
	my $patron_wants_html_flag = $patron->wants_html();

=head2 wants_ca()

This method should be used to set the wants_html flag, which indicates whether the patron desires output from the current awareness service. This is a binary attribute, and is not required. However, a devault value of '0' ('Does not want current awareness output') will be set if no value is indicated. The input to this method will be sanitized from non-binary content.

	# set a patron's wants_ca flag
	$patron->wants_ca(1);

	# get a patron's wants_ca flag
	my $patron_wants_ca_flag = $patron->wants_ca();

=head2 wants_cookie()

This method should be used to set the wants_cookie flag, which indicates whether the patron desires to have a "permanent" cookie placed on the current computer they are working on. This will allow the patron to automatically log into their MyLibrary account the next time they use this particular machine. This is a binary attribute, and is not required. However, a devault value of '0' ('Does not want permanent cookie') will be set if no value is indicated. The input to this method will be sanitized from non-binary content.

	# set a patron's wants_cookie flag
	$patron->wants_cookie(1);

	# get a patron's wants_cookie flag
	my $patron_wants_cookie_flag = $patron->wants_cookie();

=head2 username()

This method should be used to either set or get a patron's system username. The ultimate source of the username content will either come from the patron themselves or from an external authority (such as an LDAP database). This is the attribute the patron uses to identify themselves to the MyLibrary system. This is a required attribute.

	# set the patron's username
	$patron->username('johnsmith');

	# get a patron's username
	my $patron_username = $patron->username();

=head2 remote_ip()

This method can be used to set or retrieve the IP (internet protocol) address of the machine from which the user is accessing MyLibrary. This is not a required attribute. IPv4 or IPv6 addresses can be entered for this attribute. This is not a required attribute.

	# set the patron's remote IP
	$patron->remote_ip($remote_ip_address);

	# get the patron's remote IP attribute
	my $patron_IP = $patron->remote_ip();

=head2 remote_name()

This method can be used to set or retrieve the appropriately resolved host name for the computer from which the patron is accesing the MyLibrary system. This is not a required attribute.

	# set the remote host name
	$patron->remote_name('mysystem.library.nd.edu');

	# get the remote host attribute
	my $remote_patron_host_name = $patron->remote_name();


=head2 user_agent()

This method can be used to set or get the browser type that the patron uses to access the MyLibrary system. This is not a required attribute.

	# set the remote user agent type
	$patron->user_agent('Mozilla 6');

	# get the remote user agent type attribute
	my $patron_user_agent = $patron->user_agent();


=head2 last_visit()

This method can be used to get or set the date of the last time the patron visited the MyLibrary system. The input to this method will be sanitized and if an inappropriate date is input, the method will simply not execute. This is not a required attribute.

	# set the date of the last visit
	$patron->last_visit('2003-10-05');

	# get the date of the last visit
	my $patron_last_visit = $patron->last_visit();

=head2 total_visits()

This method can be used to either retrieve the total number of visits or increment the total visit count by the amount indicated. The amount indicated must be a positive integer. However, this is not a required attribute. Any other parameter input for this method will simply be ignored.

	# increment the number of total visits by a certain number
	$patron->total_visits(increment => 6);

	# retrieve the number of total visits
	my $patron_total_visits = $patron->total_visits(); 

=head2 template_id()

This method should be used to indicate which interface id the patron is associated with. An interface may be composed of several smaller components which the patron cannot choose. However, the patron may indicate a preference for a certain type of interface and this attribute links them to that interface. The parameter input for this method must be a valid integer. This is a required attribute and if no template id is indicated, a default template id will be supplied (unless the patron has previously chosen an interface).

	# supply a new interface id
	$patron->template_id(16);

	# retrieve the patron's interface id
	my $patron_template_id = $patron->template_id();

=head2 stylesheet_id()

Patrons may indicate a preference for a certain style of their interface. This will organize certain interface attributes such as coordinating colors, graphical options and positioning of interface elements. The stylesheets supplied by MyLibrary administrators will provide the patron with a choice of style for their page. This method must be used to either retrieve or set the stylesheet id with which the patron will be associated. The input to this method must be an integer. This is a required attribute. If no stylesheet id is provided, a default stylesheet will be assigned when the patron initially creates their page. However, the patron can choose another stylesheet at any time.

	# associate a stylesheet with a patron
	$patron->stylesheet_id(16);

	# retrieve the stylesheet associated with this patron
	my $patron_stylesheet_id = $patron->stylesheet_id();

=head2 commit()

This method will simply commit the current Patron object to the database and update any attribute information that has changed for an existing patron. Database integrity checks will be performed upon commit.

	# commit the Patron object to the database
	$patron->commit();

=head2 delete()

This method is used to delete a Patron object from the database. This is an irreversible process.

	# delete patron from database
	$patron->delete();

=head2 get_patrons()

This is a class method that will allow the programmer to retrieve all of the patron objects which currently exist in a MyLibrary instance. These are full class objects and any object methods can be used on the objects retrieved using this method. The method will return an array of Patron objects.

	# get all patron objects
	my @patrons = MyLibrary::Patron->get_patrons();

=head1 AUTHORS

Eric Lease Morgan <emorgan@nd.edu>
Robert Fox <rfox2@nd.edu>

=cut

sub new {
	
	# declare a few variables
	my ($class, %opts) = @_;
	my $self = {};
	
	# check for an id
	if ($opts{id}) {
	
		# find this record
		my $dbh = MyLibrary::DB->dbh();
		my $rv = $dbh->selectrow_hashref('SELECT * FROM patrons WHERE patron_id = ?', undef, $opts{id});
		if (ref($rv) eq "HASH") { $self = $rv }
		else { return }
	
	# check for username		
	} elsif ($opts{username}) {
	
		# get a record based on this username
		my $dbh = MyLibrary::DB->dbh();
		my $rv = $dbh->selectrow_hashref('SELECT * FROM patrons WHERE username = ?', undef, $opts{username});
		if (ref($rv) eq "HASH") { $self = $rv }
		else { return }
		
	}

	# return the object
	return bless $self, $class;
	
}


sub email {
	my ($self, $email) = @_;
	if ($email) { $self->{email} = $email }
	else { return $self->{email} }
}


sub name_first {
	my ($self, $name_first) = @_;
	if ($name_first) { $self->{name_f} = $name_first }
	else { return $self->{name_f} }
} 


sub name_last {
	my ($self, $name_last) = @_;
	if ($name_last) { $self->{name_l} = $name_last }
	else { return $self->{name_l} }
}


sub password {
	my ($self, $password) = @_;
	if ($password) { $self->{password} = $password }
	else { return $self->{password} }
}


sub page_title {
	my ($self, $page_title) = @_;
	if ($page_title) { $self->{page_title} = $page_title }
	else { return $self->{page_title} }
}


sub term_id {
	my ($self, $term_id) = @_;
	if ($term_id) { $self->{term_id} = $term_id }
	else { return $self->{term_id} }
}


sub telephone {
	my ($self, $telephone) = @_;
	if ($telephone) { $self->{telephone} = $telephone }
	else { return $self->{telephone} }
}


sub zipcode {
	my ($self, $zipcode) = @_;
	if ($zipcode) { $self->{zipcode} = $zipcode }
	else { return $self->{zipcode} }
}


sub address_1 {
	my ($self, $address_1) = @_;
	if ($address_1) { $self->{address_1} = $address_1 }
	else { return $self->{address_1} }
}


sub address_2 {
	my ($self, $address_2) = @_;
	if ($address_2) { $self->{address_2} = $address_2 }
	else { return $self->{address_2} }
}


sub city {
	my ($self, $city) = @_;
	if ($city) { $self->{city} = $city }
	else { return $self->{city} }
}


sub state {
	my ($self, $state) = @_;
	if ($state) { $self->{state} = $state }
	else { return $self->{state} }
}

sub browser {
	my ($self, $browser) = @_;
	if ($browser) { $self->{browser} = $browser }
	else { return $self->{browser} } 
}

sub can_contact {
	my ($self, $can_contact) = @_;
	if ($can_contact) { $self->{can_contact} = $can_contact }
	else { return $self->{can_contact} }
}

sub last_query {
	my ($self, $last_query) = @_;
	if ($last_query) { $self->{last_query} = $last_query }
	else { return $self->{last_query} }
}


sub wants_html {
	my ($self, $wants_html) = @_;
	if ($wants_html) { $self->{wants_html} = $wants_html }
	else { return $self->{wants_html} }
}

sub wants_ca {
	my ($self, $wants_ca) = @_;
	if ($wants_ca) { $self->{wants_ca} = $wants_ca }
	else { return $self->{wants_ca} }
}

sub wants_newbooks {
	my ($self, $wants_newbooks) = @_;
	if ($wants_newbooks) { $self->{wants_newbooks} = $wants_newbooks }
	else { return $self->{wants_newbooks} }
}


sub wants_cookie {
	my ($self, $wants_cookie) = @_;
	if ($wants_cookie) { $self->{wants_cookie} = $wants_cookie }
	else { return $self->{wants_cookie} }
}


sub username {
	my ($self, $username) = @_;
	if ($username) { $self->{username} = $username }
	else { return $self->{username} }
}


sub remote_ip {
	my ($self, $remote_ip) = @_;
	if ($remote_ip) { $self->{remote_ip} = $remote_ip }
	else { return $self->{remote_ip} }
}


sub remote_name {
	my ($self, $remote_name) = @_;
	if ($remote_name) { $self->{remote_name} = $remote_name }
	else { return $self->{remote_name} }
}


sub user_agent {
	my ($self, $user_agent) = @_;
	if ($user_agent) { $self->{user_agent} = $user_agent }
	else { return $self->{user_agent} }
}


sub last_visit {
	my ($self, $last_visit) = @_;
	if ($last_visit) { $self->{last_visit} = $last_visit }
	else { return $self->{last_visit} }
}


sub total_visits {
	my ($self, $total_visits) = @_;
	if ($total_visits) { $self->{total_visits} = $total_visits }
	else { return $self->{total_visits} }
}


sub template_id {
	my ($self, $template_id) = @_;
	if ($template_id) { $self->{template_id} = $template_id }
	else { return $self->{template_id} }
}


sub stylesheet_id {
	my ($self, $stylesheet_id) = @_;
	if ($stylesheet_id) { $self->{stylesheet_id} = $stylesheet_id }
	else { return $self->{stylesheet_id} }
}


sub patron_id {
	my $self = shift;
	return $self->{patron_id};
}


sub commit {

	my $self = shift;
	my $dbh = MyLibrary::DB->dbh();	

	if ($self->patron_id()) {

		my $return = $dbh->do('UPDATE patrons SET name_f = ?, name_l = ?, email =  ?, page_title = ?, term_id = ?, telephone = ?, zipcode = ?, address_1 = ?, address_2 = ?, city = ?, state = ?, can_contact = ?, password = ?, last_query = ?, wants_html = ?, wants_ca = ?, wants_cookie = ?, username = ?, remote_ip = ?, remote_name = ?, browser = ?, last_visit = ?, total_visits = ?, template_id = ?, stylesheet_id = ? WHERE patron_id = ?', undef, $self->name_first(), $self->name_last(), $self->email(), $self->page_title(), $self->term_id(), $self->telephone(), $self->zipcode(), $self->address_1(), $self->address_2(), $self->city(), $self->state(), $self->can_contact(), $self->password(), $self->last_query(), $self->wants_html(), $self->wants_ca(), $self->wants_cookie(), $self->username(), $self->remote_ip(), $self->remote_name(), $self->browser(), $self->last_visit(), $self->total_visits(), $self->template_id(), $self->stylesheet_id(), $self->patron_id());
		if ($return > 1 || $return eq undef) { croak "Patron update in commit() failed. $return records were updated."; }

	} else {

		my $id = MyLibrary::DB->nextID();		
		my $return = $dbh->do('INSERT INTO patrons (patron_id, name_f, name_l, email, page_title, term_id, telephone, zipcode, address_1, address_2, city, state, can_contact, password, last_query, wants_html, wants_ca, wants_cookie, username, remote_ip, remote_name, browser, last_visit, total_visits, template_id, stylesheet_id) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', undef, $id, $self->name_first(), $self->name_last(), $self->email(), $self->page_title(), $self->term_id(), $self->telephone(), $self->zipcode(), $self->address_1(), $self->address_2(), $self->city(), $self->state(), $self->can_contact(), $self->password(), $self->last_query(), $self->wants_html(), $self->wants_ca(), $self->wants_cookie(), $self->username(), $self->remote_ip(), $self->remote_name(), $self->browser(), $self->last_visit(), $self->total_visits(), $self->template_id(), $self->stylesheet_id());
		if ($return > 1 || $return eq undef) { croak 'Patron commit() failed.'; }
		$self->{patron_id} = $id;

	}

	return 1;

}


sub delete {

	my $self = shift;

	if ($self->patron_id()) {

		my $dbh = MyLibrary::DB->dbh();
		my $rv = $dbh->do('DELETE FROM patrons WHERE patron_id = ?', undef, $self->{patron_id});
		if ($rv != 1) {croak ("Deleted $rv records. I'll bet this isn't what you wanted.");} 
		return 1;

	}

	return 0;

}


sub get_patrons {

	my $class = shift;
	my @rv;

	my $dbh = MyLibrary::DB->dbh();
	my $patron_ids = $dbh->selectcol_arrayref('SELECT patron_id FROM patrons');
	
	foreach my $patron_id (@$patron_ids) {
	
		push (@rv, MyLibrary::Patron->new(id => $patron_id));
	
	}
	
	return @rv;
	
}


1;
